@@ -5,13 +5,19 @@ import pybreaker
 import pytest
 
 from redis.asyncio.multidb.client import MultiDBClient
+from redis.asyncio.multidb.config import InitialHealthCheck
 from redis.asyncio.multidb.database import AsyncDatabase
 from redis.asyncio.multidb.failover import WeightBasedFailoverStrategy
 from redis.asyncio.multidb.failure_detector import AsyncFailureDetector
 from redis.asyncio.multidb.healthcheck import HealthCheck
 from redis.event import EventDispatcher, AsyncOnCommandsFailEvent
 from redis.multidb.circuit import State as CBState, PBCircuitBreakerAdapter
-from redis.multidb.exception import NoValidDatabaseException
+from redis.multidb.config import DatabaseConfig
+from redis.multidb.exception import (
+    NoValidDatabaseException,
+    UnhealthyDatabaseException,
+    InitialHealthCheckFailedError,
+)
 from tests.test_asyncio.helpers import wait_for_condition
 from tests.test_asyncio.test_multidb.conftest import create_weighted_list
 
@@ -58,7 +64,7 @@ class TestMultiDbClient:
         "mock_multi_db_config,mock_db, mock_db1, mock_db2",
         [
             (
-                {},
+                {"initial_health_check_policy": InitialHealthCheck.MAJORITY_HEALTHY},
                 {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
                 {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
                 {"weight": 0.7, "circuit": {"state": CBState.OPEN}},
@@ -366,66 +372,6 @@ class TestMultiDbClient:
                 await asyncio.sleep(0.5)
                 assert await client.set("key", "value") == "OK2"
 
-    @pytest.mark.asyncio
-    @pytest.mark.parametrize(
-        "mock_multi_db_config,mock_db, mock_db1, mock_db2",
-        [
-            (
-                {},
-                {"weight": 0.2, "circuit": {"state": CBState.OPEN}},
-                {"weight": 0.7, "circuit": {"state": CBState.OPEN}},
-                {"weight": 0.5, "circuit": {"state": CBState.OPEN}},
-            ),
-        ],
-        indirect=True,
-    )
-    async def test_execute_command_throws_exception_on_failed_initialization(
-        self, mock_multi_db_config, mock_db, mock_db1, mock_db2, mock_hc
-    ):
-        databases = create_weighted_list(mock_db, mock_db1, mock_db2)
-        mock_multi_db_config.health_checks = [mock_hc]
-
-        with patch.object(mock_multi_db_config, "databases", return_value=databases):
-            mock_hc.check_health.return_value = False
-
-            client = MultiDBClient(mock_multi_db_config)
-            assert mock_multi_db_config.failover_strategy.set_databases.call_count == 1
-
-            with pytest.raises(
-                NoValidDatabaseException,
-                match="Initial connection failed - no active database found",
-            ):
-                await client.set("key", "value")
-                assert len(mock_hc.check_health.call_args_list) == 9
-
-    @pytest.mark.asyncio
-    @pytest.mark.parametrize(
-        "mock_multi_db_config,mock_db, mock_db1, mock_db2",
-        [
-            (
-                {},
-                {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
-                {"weight": 0.7, "circuit": {"state": CBState.CLOSED}},
-                {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
-            ),
-        ],
-        indirect=True,
-    )
-    async def test_add_database_throws_exception_on_same_database(
-        self, mock_multi_db_config, mock_db, mock_db1, mock_db2, mock_hc
-    ):
-        databases = create_weighted_list(mock_db, mock_db1, mock_db2)
-        mock_multi_db_config.health_checks = [mock_hc]
-
-        with patch.object(mock_multi_db_config, "databases", return_value=databases):
-            mock_hc.check_health.return_value = False
-
-            client = MultiDBClient(mock_multi_db_config)
-            assert mock_multi_db_config.failover_strategy.set_databases.call_count == 1
-
-            with pytest.raises(ValueError, match="Given database already exists"):
-                await client.add_database(mock_db)
-
     @pytest.mark.asyncio
     @pytest.mark.parametrize(
         "mock_multi_db_config,mock_db, mock_db1, mock_db2",
@@ -442,27 +388,108 @@ class TestMultiDbClient:
     async def test_add_database_makes_new_database_active(
         self, mock_multi_db_config, mock_db, mock_db1, mock_db2, mock_hc
     ):
+        """
+        Test that add_database with a DatabaseConfig creates a new database
+        and makes it active if it has the highest weight.
+        """
         databases = create_weighted_list(mock_db, mock_db2)
         mock_multi_db_config.health_checks = [mock_hc]
 
-        with patch.object(mock_multi_db_config, "databases", return_value=databases):
-            mock_db1.client.execute_command.return_value = "OK1"
-            mock_db2.client.execute_command.return_value = "OK2"
+        new_db_config = DatabaseConfig(
+            weight=0.8,  # Higher than mock_db2's 0.5
+            from_url="redis://localhost:6379",
+        )
 
-            mock_hc.check_health.return_value = True
+        with patch.object(mock_multi_db_config, "databases", return_value=databases):
+            mock_db2.client.execute_command.return_value = "OK2"
+            mock_hc.check_health = AsyncMock(return_value=True)
 
             async with MultiDBClient(mock_multi_db_config) as client:
                 assert (
                     mock_multi_db_config.failover_strategy.set_databases.call_count == 1
                 )
 
+                # Initially mock_db2 is active (highest weight among initial databases)
                 assert await client.set("key", "value") == "OK2"
-                assert len(mock_hc.check_health.call_args_list) == 6
+                initial_hc_count = len(mock_hc.check_health.call_args_list)
 
-                await client.add_database(mock_db1)
-                assert len(mock_hc.check_health.call_args_list) == 9
+                # Mock the client class to return a mock client for the new database
+                mock_new_client = AsyncMock()
+                mock_new_client.execute_command.return_value = "OK_NEW"
+                mock_new_client.connection_pool = Mock()
 
-                assert await client.set("key", "value") == "OK1"
+                with patch.object(
+                    mock_multi_db_config.client_class,
+                    "from_url",
+                    return_value=mock_new_client,
+                ):
+                    await client.add_database(new_db_config)
+
+                # Health check should have been called for the new database
+                assert len(mock_hc.check_health.call_args_list) > initial_hc_count
+
+                # New database should be active since it has highest weight
+                assert await client.set("key", "value") == "OK_NEW"
+
+    @pytest.mark.asyncio
+    @pytest.mark.parametrize(
+        "mock_multi_db_config,mock_db, mock_db1, mock_db2",
+        [
+            (
+                {},
+                {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.7, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
+            ),
+        ],
+        indirect=True,
+    )
+    async def test_add_database_skip_unhealthy_false_raises_exception(
+        self, mock_multi_db_config, mock_db, mock_db1, mock_db2, mock_hc
+    ):
+        """
+        Test that add_database with skip_unhealthy=False raises an exception
+        when the new database fails health check due to an exception.
+        """
+        databases = create_weighted_list(mock_db, mock_db2)
+        mock_multi_db_config.health_checks = [mock_hc]
+
+        new_db_config = DatabaseConfig(
+            weight=0.8,
+            from_url="redis://localhost:6379",
+        )
+
+        with patch.object(mock_multi_db_config, "databases", return_value=databases):
+            mock_db2.client.execute_command.return_value = "OK2"
+
+            # Health check returns True for existing databases, raises exception for new one
+            async def mock_check_health(database):
+                if database in [mock_db, mock_db2]:
+                    return True
+                # Raise an exception for the new database to trigger UnhealthyDatabaseException
+                raise ConnectionError("Connection refused")
+
+            mock_hc.check_health = AsyncMock(side_effect=mock_check_health)
+
+            async with MultiDBClient(mock_multi_db_config) as client:
+                # Initially mock_db2 is active
+                assert await client.set("key", "value") == "OK2"
+
+                mock_new_client = AsyncMock()
+                mock_new_client.execute_command.return_value = "OK_NEW"
+                mock_new_client.connection_pool = Mock()
+
+                with patch.object(
+                    mock_multi_db_config.client_class,
+                    "from_url",
+                    return_value=mock_new_client,
+                ):
+                    # With skip_unhealthy=False, should raise exception
+                    with pytest.raises(UnhealthyDatabaseException):
+                        await client.add_database(new_db_config, skip_unhealthy=False)
+
+                # Database list should remain unchanged
+                assert len(client.get_databases()) == 2
 
     @pytest.mark.asyncio
     @pytest.mark.parametrize(
@@ -684,3 +711,216 @@ class TestMultiDbClient:
                     match="Cannot set active database, database is unhealthy",
                 ):
                     await client.set_active_database(mock_db1)
+
+
+@pytest.mark.onlynoncluster
+class TestInitialHealthCheckPolicy:
+    """Tests for initial health check policy evaluation."""
+
+    @pytest.mark.asyncio
+    @pytest.mark.parametrize(
+        "mock_multi_db_config,mock_db, mock_db1, mock_db2",
+        [
+            (
+                {},
+                {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.7, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
+            ),
+        ],
+        indirect=True,
+    )
+    async def test_all_healthy_policy_succeeds_when_all_databases_healthy(
+        self, mock_multi_db_config, mock_db, mock_db1, mock_db2, mock_hc
+    ):
+        """
+        Test that ALL_HEALTHY policy succeeds when all databases pass health check.
+        """
+        databases = create_weighted_list(mock_db, mock_db1, mock_db2)
+        mock_multi_db_config.health_checks = [mock_hc]
+
+        with patch.object(mock_multi_db_config, "databases", return_value=databases):
+            mock_db1.client.execute_command.return_value = "OK1"
+            mock_hc.check_health = AsyncMock(return_value=True)
+
+            async with MultiDBClient(mock_multi_db_config) as client:
+                # Should succeed without raising InitialHealthCheckFailedError
+                assert await client.set("key", "value") == "OK1"
+
+    @pytest.mark.asyncio
+    @pytest.mark.parametrize(
+        "mock_multi_db_config,mock_db, mock_db1, mock_db2",
+        [
+            (
+                {},
+                {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.7, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
+            ),
+        ],
+        indirect=True,
+    )
+    async def test_all_healthy_policy_fails_when_one_database_unhealthy(
+        self, mock_multi_db_config, mock_db, mock_db1, mock_db2, mock_hc
+    ):
+        """
+        Test that ALL_HEALTHY policy fails when any database fails health check.
+        """
+        databases = create_weighted_list(mock_db, mock_db1, mock_db2)
+        mock_multi_db_config.health_checks = [mock_hc]
+
+        with patch.object(mock_multi_db_config, "databases", return_value=databases):
+
+            async def mock_check_health(database):
+                # mock_db2 is unhealthy
+                return database != mock_db2
+
+            mock_hc.check_health = AsyncMock(side_effect=mock_check_health)
+
+            with pytest.raises(
+                InitialHealthCheckFailedError,
+                match="Initial health check failed",
+            ):
+                async with MultiDBClient(mock_multi_db_config) as client:
+                    await client.set("key", "value")
+
+    @pytest.mark.asyncio
+    @pytest.mark.parametrize(
+        "mock_multi_db_config,mock_db, mock_db1, mock_db2",
+        [
+            (
+                {"initial_health_check_policy": InitialHealthCheck.MAJORITY_HEALTHY},
+                {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.7, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
+            ),
+        ],
+        indirect=True,
+    )
+    async def test_majority_healthy_policy_succeeds_when_majority_healthy(
+        self, mock_multi_db_config, mock_db, mock_db1, mock_db2, mock_hc
+    ):
+        """
+        Test that MAJORITY_HEALTHY policy succeeds when more than half of databases are healthy.
+        With 3 databases, 2 healthy is a majority.
+        """
+        databases = create_weighted_list(mock_db, mock_db1, mock_db2)
+        mock_multi_db_config.health_checks = [mock_hc]
+
+        with patch.object(mock_multi_db_config, "databases", return_value=databases):
+            mock_db1.client.execute_command.return_value = "OK1"
+
+            async def mock_check_health(database):
+                # mock_db2 is unhealthy, but 2 out of 3 are healthy (majority)
+                return database != mock_db2
+
+            mock_hc.check_health = AsyncMock(side_effect=mock_check_health)
+
+            async with MultiDBClient(mock_multi_db_config) as client:
+                # Should succeed - 2 out of 3 healthy is a majority
+                assert await client.set("key", "value") == "OK1"
+
+    @pytest.mark.asyncio
+    @pytest.mark.parametrize(
+        "mock_multi_db_config,mock_db, mock_db1, mock_db2",
+        [
+            (
+                {"initial_health_check_policy": InitialHealthCheck.MAJORITY_HEALTHY},
+                {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.7, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
+            ),
+        ],
+        indirect=True,
+    )
+    async def test_majority_healthy_policy_fails_when_minority_healthy(
+        self, mock_multi_db_config, mock_db, mock_db1, mock_db2, mock_hc
+    ):
+        """
+        Test that MAJORITY_HEALTHY policy fails when less than half of databases are healthy.
+        With 3 databases, only 1 healthy is not a majority.
+        """
+        databases = create_weighted_list(mock_db, mock_db1, mock_db2)
+        mock_multi_db_config.health_checks = [mock_hc]
+
+        with patch.object(mock_multi_db_config, "databases", return_value=databases):
+
+            async def mock_check_health(database):
+                # Only mock_db is healthy (1 out of 3 is not a majority)
+                return database == mock_db
+
+            mock_hc.check_health = AsyncMock(side_effect=mock_check_health)
+
+            with pytest.raises(
+                InitialHealthCheckFailedError,
+                match="Initial health check failed",
+            ):
+                async with MultiDBClient(mock_multi_db_config) as client:
+                    await client.set("key", "value")
+
+    @pytest.mark.asyncio
+    @pytest.mark.parametrize(
+        "mock_multi_db_config,mock_db, mock_db1, mock_db2",
+        [
+            (
+                {"initial_health_check_policy": InitialHealthCheck.ANY_HEALTHY},
+                {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.7, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
+            ),
+        ],
+        indirect=True,
+    )
+    async def test_any_healthy_policy_succeeds_when_one_database_healthy(
+        self, mock_multi_db_config, mock_db, mock_db1, mock_db2, mock_hc
+    ):
+        """
+        Test that ANY_HEALTHY policy succeeds when at least one database is healthy.
+        """
+        databases = create_weighted_list(mock_db, mock_db1, mock_db2)
+        mock_multi_db_config.health_checks = [mock_hc]
+
+        with patch.object(mock_multi_db_config, "databases", return_value=databases):
+            mock_db.client.execute_command.return_value = "OK"
+
+            async def mock_check_health(database):
+                # Only mock_db is healthy
+                return database == mock_db
+
+            mock_hc.check_health = AsyncMock(side_effect=mock_check_health)
+
+            async with MultiDBClient(mock_multi_db_config) as client:
+                # Should succeed - at least one database is healthy
+                assert await client.set("key", "value") == "OK"
+
+    @pytest.mark.asyncio
+    @pytest.mark.parametrize(
+        "mock_multi_db_config,mock_db, mock_db1, mock_db2",
+        [
+            (
+                {"initial_health_check_policy": InitialHealthCheck.ANY_HEALTHY},
+                {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.7, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
+            ),
+        ],
+        indirect=True,
+    )
+    async def test_any_healthy_policy_fails_when_no_database_healthy(
+        self, mock_multi_db_config, mock_db, mock_db1, mock_db2, mock_hc
+    ):
+        """
+        Test that ANY_HEALTHY policy fails when no database is healthy.
+        """
+        databases = create_weighted_list(mock_db, mock_db1, mock_db2)
+        mock_multi_db_config.health_checks = [mock_hc]
+
+        with patch.object(mock_multi_db_config, "databases", return_value=databases):
+            mock_hc.check_health = AsyncMock(return_value=False)
+
+            with pytest.raises(
+                InitialHealthCheckFailedError,
+                match="Initial health check failed",
+            ):
+                async with MultiDBClient(mock_multi_db_config) as client:
+                    await client.set("key", "value")
