@@ -6,6 +6,7 @@ import pytest
 
 from redis.asyncio.client import Pipeline
 from redis.asyncio.multidb.client import MultiDBClient
+from redis.asyncio.multidb.config import InitialHealthCheck
 from redis.asyncio.multidb.failover import WeightBasedFailoverStrategy
 from redis.multidb.circuit import State as CBState, PBCircuitBreakerAdapter
 from tests.test_asyncio.helpers import wait_for_condition
@@ -68,7 +69,7 @@ class TestPipeline:
         "mock_multi_db_config,mock_db, mock_db1, mock_db2",
         [
             (
-                {},
+                {"initial_health_check_policy": InitialHealthCheck.MAJORITY_HEALTHY},
                 {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
                 {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
                 {"weight": 0.7, "circuit": {"state": CBState.OPEN}},
@@ -324,7 +325,7 @@ class TestTransaction:
         "mock_multi_db_config,mock_db, mock_db1, mock_db2",
         [
             (
-                {},
+                {"initial_health_check_policy": InitialHealthCheck.MAJORITY_HEALTHY},
                 {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
                 {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
                 {"weight": 0.7, "circuit": {"state": CBState.OPEN}},
